#!/usr/bin/python
#coding=utf-8
import os.path 
import sys
import ConfigParser

class Config(object):

    data = {}
    cookies = {}

    def __init__(self, path = os.path.expanduser("~") + '/.pybili.conf'):
        if not os.path.isfile(path):
            print path
            print 'warning! please set the config file!'
            return 
        self.path = path
        self.read(path)

    def get(self, dataid, key, value = None):
        if not str(dataid) in self.data: return value
        d = self.data[str(dataid)]
        v = d.get(key, None)
        if not v: return value
        return v

    def _get(self, cf, sec, opt):
        if cf.has_option(sec, opt): return cf.get(sec, opt)
        else: return

    def _getboolean(self, cf, sec, opt):
        if cf.has_option(sec, opt): return cf.getboolean(sec, opt)
        else: return
    
    def read(self, path):
        cf = ConfigParser.ConfigParser(allow_no_value=1)
        cf.read(path)
        
        for section in cf.sections():
            if section == 'cookies':
                s = cf.get(section, 'cookies')
                if 'cookie:' == s[:7]: s = s[7:] # do strip 
                try:
                    for line in s.split(';'):
                        name, value = line.strip().split('=', 1)
                        self.cookies[name] = value
                except Exception, e:
                    print 'warning! please set correct cookies into \'cookie.txt\'', e
            else:
                try:
                    d = {}
                    d['GiftResponse'] = self._getboolean(cf, section, 'GiftResponse')
                    d['ShowTime'] = self._getboolean(cf, section, 'ShowTime')
                    d['SmallTVHint'] = self._getboolean(cf, section, 'SmallTVHint')
                    d['MacNotification'] = self._getboolean(cf, section, 'MacNotification')
                    d['MacTTS'] = self._getboolean(cf, section, 'MacTTS')
                    d['DanmakuColor'] = self._get(cf, section, 'DanmakuColor')
                    d['AwardSmallTV'] = self._getboolean(cf, section, 'AwardSmallTV')
                    d['RecordDanmaku'] = self._getboolean(cf, section, 'RecordDanmaku')
                    self.data[section] = d
                except Exception, e:
                    print 'exception when read config, please check the format of config file ', e

    def p(self):
        print 'current configs:'
        for k in self.data.keys():
            print '----------'
            print k
            for kk in self.data[k].keys():
                print kk, self.data[k][kk]

    def inputBoolean(self, msg):
        a = raw_input(msg)
        return a in 'Yy1'

    def setup(self):
        cf = ConfigParser.RawConfigParser()
        cf.read(self.path)

        cookies = raw_input('Please input the cookies: [press enter to skip]')
        if cookies:
            if not cf.has_section('cookies'): cf.add_section('cookies')
            cf.set('cookies', 'cookies', cookies)

        roomid = raw_input('Please input the roomid to setup: [press enter to skip]')
        if roomid:
            if not cf.has_section(roomid): cf.add_section(roomid)

            b = self.inputBoolean('Auto-response the gift? [y/n]')
            cf.set(roomid, 'GiftResponse', b)
            b = self.inputBoolean('Show the time of the danmaku in console? [y/n]')
            cf.set(roomid, 'ShowTime', b)
            b = self.inputBoolean('Auto-response the smallTV? [y/n]')
            cf.set(roomid, 'SmallTVHint', b)
            b = self.inputBoolean('Auto-enter the lottery of the smallTV? [y/n]')
            cf.set(roomid, 'AwardSmallTV', b)
            b = self.inputBoolean('Show notification in the Mac OS? [y/n]')
            cf.set(roomid, 'MacNotification', b)
            b = self.inputBoolean('Speak the danmaku out in the Mac OS? [y/n]')
            cf.set(roomid, 'MacTTS', b)
            b = self.inputBoolean('Record the danmaku into the MongoDB? [y/n]')
            cf.set(roomid, 'RecordDanmaku', b)
            cf.set(roomid, 'DanmakuColor', raw_input('Danmaku color? [white/red/orange/yellow/green/cyan/blue/purple]'))
        cf.write(open(self.path, 'w'))

def main():
    config = Config()
    config.p()
    config.setup()

if __name__ == '__main__':
    main()



#!/usr/bin/python
#coding=utf-8
import requests
import json
import time 
import sys
import random

SEND_URL = 'http://live.bilibili.com/msg/send'

class Sender(object):

    def __init__(self, cookies):
        self.cookies = cookies

    def sendDanmaku(self, roomid, content, color='white'):
        try:
            content = content.strip()
            if not content: return
            if color == 'blue': color = 6737151
            elif color == 'green': color = 8322816
            else: color = 16777215 # white
            params = {
                "color":color, 
                "fontsize":25,
                "mode":1,
                "msg":content,
                "rnd":int(time.time()),
                "roomid":roomid
                }
            r = requests.post(SEND_URL, data=params, cookies=self.cookies)
            result = r.content
            raw = json.loads(result)
            if raw['code'] != 0: print raw['msg']
        except:
            print 'send danmaku error:', sys.exc_info()[0]
    

lst = '''
        塞纳河畔 左岸的咖啡
我手一杯 品尝你的美
留下唇印的嘴
花店玫瑰 名字写错谁
告白气球 风吹到对街
微笑在天上飞
你说你有点难追 想让我知难而退
礼物不需挑最贵 只要香榭的落叶
喔 营造浪漫的约会 不害怕搞砸一切
拥有你就拥有 全世界
亲爱的 爱上你 从那天起
甜蜜的很轻易
亲爱的 别任性 你的眼睛
在说我愿意
塞纳河畔 左岸的咖啡
我手一杯 品尝你的美
留下唇印的嘴
花店玫瑰 名字写错谁
告白气球 风吹到对街
微笑在天上飞
你说你有点难追 想让我知难而退
礼物不需挑最贵 只要香榭的落叶
喔 营造浪漫的约会 不害怕搞砸一切
拥有你就拥有 全世界
亲爱的 爱上你 从那天起
甜蜜的很轻易
亲爱的 别任性 你的眼睛
在说我愿意
亲爱的 爱上你 恋爱日记
飘香水的回忆
一整瓶 的梦境 全都有你
搅拌在一起
亲爱的 别任性 你的眼睛
在说我愿意
5片式镜头 ƒ/2.0 光圈
双色温闪光灯
支持 PDAF 相位对焦
暗光画质增强技术
HDR高动态范围调节技术
全景模式
连拍模式
面部识别功能
实时滤镜拍照
第二代36级智能美颜 自拍实时美颜
ƒ/2.0 大光圈 85°大广角
视频通话实时美颜
倒计时自拍
面部识别功能
支持魔镜功能 实时评测颜值
支持阳光屏 强烈阳光下清晰可见屏幕内容
支持夜光屏
支持护眼模式
支持色温调节
        '''.split()

cks = ["sid=cohdat21; fts=1497021168; LIVE_BUVID=146ada79a17e6839cbe254b809986f8e; LIVE_BUVID__ckMd5=deef7b5e401f1ac0; buvid3=3C5D4495-EA05-4335-80FB-8406E4F99B9A15603infoc; DedeUserID=28919882; DedeUserID__ckMd5=6025d570808b12d6; SESSDATA=7d7aee87%2C1499613190%2Cc44752f8; bili_jct=5664bee6709796edbf715b751a6c2061; LIVE_LOGIN_DATA=c27d7350e7c33b0afd725b7230ebdaa5afe00ae6; LIVE_LOGIN_DATA__ckMd5=cd66751c50b04b60; _dfcaptcha=bc59ef133c7755f6d60e9cb24a65768a; attentionData=%7B%22code%22%3A0%2C%22msg%22%3A%22success%22%2C%22message%22%3A%22success%22%2C%22data%22%3A%7B%22count%22%3A0%2C%22open%22%3A1%2C%22has_new%22%3A0%7D%7D; Hm_lvt_8a6e55dbd2870f0f5bc9194cddf32a02=1497021168; Hm_lpvt_8a6e55dbd2870f0f5bc9194cddf32a02=1497021198; F_S_T_28919882=1; user_face=http%3A%2F%2Fi1.hdslb.com%2Fbfs%2Fface%2Fb11d5c14c1445c1b5093e2dcad51d58221282bc7.jpg",
        "sid=j44742hb; fts=1497021328; LIVE_BUVID=067930dfaf06c68c46ddc766599a7198; LIVE_BUVID__ckMd5=dfe68c58efb81910; buvid3=FC19C6FE-314D-4982-AFE2-096CD689581032373infoc; DedeUserID=15653302; DedeUserID__ckMd5=2740846aa032723c; SESSDATA=641a6b54%2C1499613344%2C1eaef350; bili_jct=1e23321cfdfaebc7f0c9b40324cede1d; LIVE_LOGIN_DATA=40db80532a9f5fda6e558afbefae27900f3866e1; LIVE_LOGIN_DATA__ckMd5=2ae63537cd546605; _dfcaptcha=5e52181bcfb3bf52961c3e28fc079a90; attentionData=%7B%22code%22%3A0%2C%22msg%22%3A%22success%22%2C%22message%22%3A%22success%22%2C%22data%22%3A%7B%22count%22%3A0%2C%22open%22%3A1%2C%22has_new%22%3A0%7D%7D; user_face=http%3A%2F%2Fi2.hdslb.com%2Fbfs%2Fface%2F426eca544511d6416f2c7f6c7c3e78bdd6e50579.jpg; F_S_T_15653302=1; Hm_lvt_8a6e55dbd2870f0f5bc9194cddf32a02=1497021329; Hm_lpvt_8a6e55dbd2870f0f5bc9194cddf32a02=1497021352", 
        "sid=la5zn8lk; fts=1497023179; LIVE_BUVID=e77111075c93a9d3c94622dacdcc0505; LIVE_BUVID__ckMd5=51343520b0568123; buvid3=A5F3E262-3402-4605-92E5-255DDF59F4C732358infoc; DedeUserID=35076534; DedeUserID__ckMd5=2864d5fe0301aff3; SESSDATA=9a737022%2C1499615330%2C84b2a1d4; bili_jct=a5f038c4f44d83593ff8f013b9186fba; LIVE_LOGIN_DATA=f537448b1a8480a667edce29e95896d76a95a461; LIVE_LOGIN_DATA__ckMd5=ba6f130d402af57f; _dfcaptcha=c802f04dc6380cc5b660414d50d54ebc; attentionData=%7B%22code%22%3A0%2C%22msg%22%3A%22success%22%2C%22message%22%3A%22success%22%2C%22data%22%3A%7B%22count%22%3A0%2C%22open%22%3A1%2C%22has_new%22%3A0%7D%7D; Hm_lvt_8a6e55dbd2870f0f5bc9194cddf32a02=1497023180; Hm_lpvt_8a6e55dbd2870f0f5bc9194cddf32a02=1497023338; F_S_T_35076534=1; user_face=http%3A%2F%2Fi1.hdslb.com%2Fbfs%2Fface%2Fbdd22abffb6847ca6398365c52eeecf7ac171674.jpg"]

def parseCookie(s):
    c = {}
    s = s.strip()
    if 'cookie:' == s[:7]: s = s[7:] # do strip 
    try:
        for line in s.split(';'):
            name, value = line.strip().split('=', 1)
            c[name] = value
    except:
        pass
    return c

def main():
    import bili_config
    date_format = '%m%d-%H%M%S'
    config = bili_config.Config()
    senders = [Sender(config.cookies)]
    for c in map(parseCookie, cks):
        print c
        senders += [Sender(c)]

    #for s in lst: print s
    print "danmakus count:", len(lst)
    while 1:
        tm = time.localtime(time.time() + 18000)
        mins = int(time.strftime('%M', tm))
        hours = int(time.strftime('%H', tm))
        if 11 <= hours <= 23:
            if 50 <= mins < 60:
                content = random.choice(lst)
                for sender in senders:
                    sender.sendDanmaku(545342, content)
                    #sender.sendDanmaku(90012, "笔芯")
                    print time.strftime(date_format, tm), content
                    time.sleep(30)
            elif mins == 49:
                print time.strftime(date_format, tm)
                time.sleep(10)
            else:
                print time.strftime(date_format, tm)
                time.sleep(60)
        else:
            time.sleep(1800)

if __name__ == '__main__':
    main()



#!/usr/bin/python
#coding=utf-8
import time
import json
import thread
import threading
import subprocess
import re

from bili import DanmakuHandler
from bili import SimpleDanmakuHandler
from pymongo import MongoClient

DEBUG = 0

class MongoHandler(DanmakuHandler):

    def __init__(self):
        try:
            print 'start db logger...'
            self.client = MongoClient()
            self.db = self.client.danmaku
        except:
            print 'mongodb service down!\nto install:[brew install mongodb]\nto start:[brew services start mongodb]'

    def handleDanmaku(self, danmaku):
        try:
            self.insert(danmaku)
        except:
            pass

    def insert(self, danmaku):
        danmaku.roomid = self.roomid
        self.db.raw.insert_one({
            'roomid':danmaku.roomid,
            'action':danmaku.action,
            'raw':danmaku.rawData,
            'time':time.strftime('%y%m%d-%H%M%S', danmaku.time)
            })

        if hasattr(danmaku, 'user') and hasattr(danmaku, 'text'):
            self.db.plain.insert_one({
                'roomid':danmaku.roomid,
                'user':danmaku.user,
                'text':danmaku.text,
                'time':time.strftime('%y%m%d-%H%M%S', danmaku.time)
                })

class TTSHandler(SimpleDanmakuHandler):

    def __init__(self):
        print 'start macTTS...'
        self.sayLOCK = threading.Lock()
        self.saySentences = []
        self.startTTSThread()
    
    def startTTSThread(self):
        thread.start_new_thread(self.sayThread, ())

    def handleDanmaku(self, danmaku):
        super(TTSHandler, self).handleDanmaku(danmaku)
        if hasattr(danmaku, 'text'):
           self.toSay(danmaku.text)

    def say(self, s):
        t = re.sub(r'23[3]+', ' 2 3 3 ', s)
        t = re.sub(r'66[6]+', ' 6 6 6 ', t)
        t = re.sub(r'hh[h]+', ' 哈哈哈 ', t)
        t = t.replace('(°∀°)ﾉ', '挥手')
        t = t.replace('(╯‵□′)╯︵┻━┻', '掀桌')
        t = t.replace('°', ' ')
        t = t.replace('-', ' ')
        cmd = 'say \'%s\'' % (t)
        if DEBUG: print cmd
        self.tts_s = subprocess.Popen(cmd, shell=True)
        self.tts_s.wait()

    def toSay(self, s):
        self.sayLOCK.acquire()
        self.saySentences = [s] + self.saySentences
        self.sayLOCK.release()
    
    def sayThread(self):
        while 1:
            while self.saySentences:
                self.sayLOCK.acquire()
                s = self.saySentences.pop()
                self.sayLOCK.release()
                self.say(s)
            time.sleep(1)

class NotifcationHandler(SimpleDanmakuHandler):

    def __init__(self):
        print 'start macNotification...'
        self.notificationTimers = {}

    def handleDanmaku(self, danmaku):
        super(NotifcationHandler, self).handleDanmaku(danmaku)
        if hasattr(danmaku, 'user') & hasattr(danmaku, 'text'):
           self.showMacNotification(danmaku.user, danmaku.text)

    def showMacNotification(self, title, content):
        title = title.replace('\'', ' ')
        content = content.replace('\'', ' ')
        content = content.replace('-', ' ')
        content = content.replace('(T_T)', '小哭脸')
        cmd = "terminal-notifier -title '%s' -message '%s' -group '%s' > /dev/null" % (title, content, title)
        subprocess.Popen(cmd, shell=True)
        t = threading.Timer(10, self.closeMacNotification, (title, ))
        t.start()
        if title in self.notificationTimers: self.notificationTimers[title].cancel()
        self.notificationTimers[title] = t

    def closeMacNotification(self, group):
        cmd = "terminal-notifier -remove '%s' > /dev/null" % group
        subprocess.Popen(cmd, shell=True)

class GiftResponseHandler(SimpleDanmakuHandler):

    def __init__(self, sender):
        print 'start gift response...'
        self.startGiftResponseThread()
        self.gifts = []
        self.sender = sender
        self.LOCK = threading.Lock()

    def handleDanmaku(self, danmaku):
        super(GiftResponseHandler, self).handleDanmaku(danmaku)
        if hasattr(danmaku, 'json'):
            raw = danmaku.json
            if raw['cmd'] == 'SEND_GIFT':
                data = raw['data']
                uname, num, giftName = data['uname'].encode('utf-8'), data['num'], data['giftName'].encode('utf-8')
                self.LOCK.acquire()
                self.gifts += [(uname, num, giftName)]
                self.LOCK.release()
        
    def giftResponseThread(self):
        while 1:
            self.LOCK.acquire()
            if self.gifts:
                uname, num, giftName = self.gifts[0]
                for t_uname, t_num, t_giftName in self.gifts[1:]:
                    num = '好多'
                    if t_uname != uname: uname = '大家'
                    if t_giftName != giftName: giftName = '礼物'
                self.sender.sendDanmaku(self.roomid, '谢谢%s送的%s个%s' % (uname, str(num), giftName))
                while self.gifts: self.gifts.pop()
            self.LOCK.release()
            time.sleep(2)

    def startGiftResponseThread(self):
        thread.start_new_thread(self.giftResponseThread, ())

class RewardResponseHandler(SimpleDanmakuHandler):

    def __init__(self, sender):
        print 'start smallTV response...'
        self.sender = sender
        self.date_format = '%H:%M:%S'

    def handleDanmaku(self, danmaku):
        super(RewardResponseHandler, self).handleDanmaku(danmaku)
        if hasattr(danmaku, 'json'):
            raw = danmaku.json
            if raw['cmd'] in ['SYS_MSG']: 
                if 'roomid' in raw:
                    roomid = str(raw['roomid'])
                    tm = time.strftime(self.date_format, time.localtime(time.time() + 180 + 18000))
                    self.sender.sendDanmaku(self.roomid, '%s房间小电视啦，有效期至%s' % (roomid, tm))

class AutoRewardHandler(SimpleDanmakuHandler):

    def __init__(self, sender):
        print 'start auto reward smallTV...'
        self.sender = sender

    def handleDanmaku(self, danmaku):
        super(AutoRewardHandler, self).handleDanmaku(danmaku)
        if hasattr(danmaku, 'json'):
            raw = danmaku.json
            if raw['cmd'] in ['SYS_MSG']: 
                if 'roomid' in raw:
                    roomid = str(raw['roomid'])
                    self.sender.joinSmallTV(roomid, raw['tv_id'])

def main():
    pass

if __name__ == '__main__':
    main()



#!/usr/bin/python
#coding=utf-8
import socket
import struct
import random
import json
import thread
import time
import sys

reload(sys)  
sys.setdefaultencoding('utf-8')

HOST = 'livecmt-2.bilibili.com'
PORT = 788
BUFFER_SIZE = 128 * 1024

DEBUG = 0

class Danmaku(object):
    def __init__(self, action, rawData, time):
        self.action = action
        self.rawData = rawData
        self.time = time

class DanmakuHandler(object):

    def setRoomId(self, roomid):
        self.roomid = roomid

    def handleDanmaku(self, danmaku):
        pass

class SimpleDanmakuHandler(DanmakuHandler):

    def handleDanmaku(self, danmaku):
        if hasattr(danmaku, 'json'): return
        body = danmaku.rawData
        if not body: return
        if danmaku.action == 5:
            raw = json.loads(body)
            danmaku.json = raw
            if 'info' in raw:
                info = raw['info']
                user, content = info[2][1].encode('utf-8'), info[1].encode('utf-8')
                danmaku.user = user
                danmaku.text = content
                danmaku.isManager = info[2][2]

class BiliHelper(object):

    def __init__(self, roomid, *packetHandlers):
        print '----------------------------'
        print '| bili danmaku helper v0.2 |' 
        print '----------------------------'
        self.lastPacket = 0
        self.roomid = roomid
        self.danmakuHandlers = packetHandlers
        for handler in packetHandlers: handler.setRoomId(roomid)
        self.heartBeatThreadAlive, self.packetReceiveThreadAlive = 0, 0
        self.startHeartBeatThread(), self.startPacketReceiveThread()
        time.sleep(5) # wait 5s for thread start
        thread.start_new_thread(self.localCheckThread, ())

    def startHeartBeatThread(self):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((HOST, PORT))
        self.joinChannel(self.roomid)
        if not self.heartBeatThreadAlive:
            thread.start_new_thread(self.heartBeatThread, ())
            self.heartBeatThreadAlive = True
            if DEBUG: print 'heartBeatThreadStarted!'

    def startPacketReceiveThread(self):
        if not self.packetReceiveThreadAlive:
            thread.start_new_thread(self.recvThread, ())
            self.packetReceiveThreadAlive = True
            if DEBUG: print 'packetReceiveThreadStarted!'

    def sendPacket(self, action, body):
        playload = body.encode('utf-8')
        packetLength = len(playload) + 16
        head = struct.pack('>IHHII', packetLength, 16, 1, action, 1)
        self.s.send(head + playload)

    def joinChannel(self, channelId, uid = int(1e14 + 2e14 * random.random())):
        body = json.dumps({'roomid':channelId, 'uid': uid}, separators=(',',':'))
        self.sendPacket(7, body)

    def localCheckThread(self):
        while 1:
            try:
                if not self.heartBeatThreadAlive:
                    if DEBUG: print 'detect heartBeatThread down, restarting...'
                    self.startHeartBeatThread()
                if not self.packetReceiveThreadAlive:
                    if DEBUG: print 'detect packetReceiveThread down, restarting...'
                    self.startPacketReceiveThread()
            except:
                pass
            time.sleep(5)

    def heartBeatThread(self):
        try:
            while 1:
                self.sendPacket(2, '')
                time.sleep(30)
        except Exception, e:
            if DEBUG: print 'heartBeatThread down!'
            self.heartBeatThreadAlive = False

    def handleUnfinishedPacket(self, packet):
        if self.lastPacket: 
            if DEBUG: 
                print '--------------------------'
                print 'last unfinished packet, ', repr(packet)
                print '--------------------------'
            if len(packet) < 16: return packet
            header = struct.unpack('>IHHII', packet[:16])
            if header[0] > 0xffff:
                packet = self.lastPacket + packet
                if DEBUG: print 'contact last packet, ', repr(packet)
            self.lastPacket = 0
        return packet

    def parsePacket(self, packet):
        packet = self.handleUnfinishedPacket(packet)
        try:
            while packet:
                if len(packet) < 16: return
                header = struct.unpack('>IHHII', packet[:16])
                packetLength = int(header[0])
                body = packet[16:packetLength]
                action = header[3]
                danmaku = Danmaku(action, body, time.localtime(time.time()))

                if packetLength > len(packet):
                    if DEBUG:
                        print 'packetLengthError!', header, len(packet), repr(packet[:packetLength]), repr(packet[packetLength:])
                    self.lastPacket = packet
                    break
                for handler in self.danmakuHandlers: handler.handleDanmaku(danmaku)
                packet = packet[packetLength:]
        except SystemExit:  
            return  
        except Exception, e:
            print 'decode error!', e

    def recvThread(self):
        try:
            while 1:
                packet = self.s.recv(BUFFER_SIZE)
                if packet: self.parsePacket(packet)
        except Exception, e:
            if DEBUG: print 'recvThread down!', e
            self.packetReceiveThreadAlive = False

if __name__ == '__main__':
    pass



#!/usr/bin/python
#coding=utf-8
import requests
import json
import time 
import sys

SEND_URL = 'http://live.bilibili.com/msg/send'
TV_URL = 'http://api.live.bilibili.com/SmallTV/join'
QUERY_NEED_YOU = 'http://api.live.bilibili.com/activity/v1/NeedYou/getLiveInfo'
NEED_YOU_URL = 'http://api.live.bilibili.com/activity/v1/NeedYou/getLiveAward'

class Sender(object):

    def __init__(self, cookies):
        self.cookies = cookies
        self.lightenIds = set()

    def sendDanmaku(self, roomid, content, color='white'):
        try:
            content = content.strip()
            if not content: return
            if color == 'blue': color = 6737151
            elif color == 'green': color = 8322816
            else: color = 16777215 # white
            params = {
                "color":color, 
                "fontsize":25,
                "mode":1,
                "msg":content,
                "rnd":int(time.time()),
                "roomid":roomid
                }
            r = requests.post(SEND_URL, data=params, cookies=self.cookies)
            result = r.content
            raw = json.loads(result)
            if raw['code'] != 0: print raw['msg']
            return True
        except:
            return False

    
    def joinSmallTV(self, roomid, tv_id):
        params = {
                'roomid':roomid,
                'id':tv_id,
                '_':int(time.time() * 100)
                }
        r = requests.get(TV_URL, params=params, cookies=self.cookies)
        result = r.content
        raw = json.loads(result)
        if raw['code'] != 0: print raw['msg']

    def _joinNeedYou(self, roomid, lightenId):
        params = {
                'roomid':roomid,
                'lightenId':lightenId
                }
        r = requests.post(NEED_YOU_URL, data=params, cookies=self.cookies)
        result = r.content
        raw = json.loads(result)
        #if raw['code'] != 0: print raw['msg']
        #print raw['msg']

    def joinNeedYou(self, roomid):
        params = {
                'roomid':roomid
                }
        r = requests.get(QUERY_NEED_YOU, params=params, cookies=self.cookies)
        result = r.content
        raw = json.loads(result)
        if raw['code'] == 0 and 'lightenId' in result:
            for d in raw['data']:
                lightenId = d['lightenId']
                if lightenId not in self.lightenIds:
                    self._joinNeedYou(roomid, lightenId)
                    self.lightenIds.add(lightenId)

        #if raw['code'] != 0: print raw['msg']
        #print self.lightenIds

def main():
    import bili_config
    argv = sys.argv
    config = bili_config.Config()
    sender = Sender(config.cookies)
    while 1:
        content = raw_input()
        sender.sendDanmaku(int(argv[1]), content)

if __name__ == '__main__':
    main()



#!/usr/bin/env python      
import sys
import Tkinter as tk       
import DanMuJi
import bili
import re
import emoji_list

DEBUG = 0
HEIGHT = 32

class Application(tk.Frame):              
    def __init__(self, master=None):
        tk.Frame.__init__(self, master)   
        self.grid()                       
        self.createWidgets()

    def createWidgets(self):
        self.w = tk.Text(self, height=HEIGHT, font=("simhei", 20))
        self.w.pack(expand=tk.YES, fill=tk.BOTH)
        self.w.grid()

        self.quitButton = tk.Button(self, text='Quit',
            command=self.quit)            
        self.quitButton.grid()            

class GUIDanmakuHandler(bili.SimpleDanmakuHandler):

    def __init__(self, w):
        self.w = w
        self.cnt = 0
        self.all_emoji = emoji_list.all_emoji

    def stripEmoji(self, s):
        for e in self.all_emoji:
            if e not in '0123456789#*': s = s.replace(e, '')
        return s

    def handleDanmaku(self, danmaku):
        super(GUIDanmakuHandler, self).handleDanmaku(danmaku)
        if hasattr(danmaku, 'user') & hasattr(danmaku, 'text'):
            if self.cnt > HEIGHT * 10: self.w.delete(1.0, 2.0)
            else: self.cnt += 1
            user = self.stripEmoji(danmaku.user)
            text = self.stripEmoji(danmaku.text)
            if text and user: self.w.insert(tk.END, '%s : %s\n' % (user, text)) 
            self.w.see(tk.END)

def main():
    argv = sys.argv
    roomid = 90012
    if len(argv) == 2: roomid = int(argv[1])

    app = Application()                      
    danmakuHandlers = DanMuJi.initHandlers(roomid) + [GUIDanmakuHandler(app.w)]
    bili.BiliHelper(roomid, *danmakuHandlers)
    app.master.title('bilibili danmaku helper')
    app.mainloop()                           
    
if __name__ == '__main__':
    main()



#!/usr/bin/python
#coding=utf-8
import bili
import sys
import thread
import struct
import time
import json

reload(sys)  
sys.setdefaultencoding('utf8')

DEBUG = 1

class DanmakuHandler(bili.DanmakuHandler):
    
    block_lst = ['圆舞曲', '肖邦', '钟']
    to_play_lst = []
    pending_lst = []
    completed = {}

    def __init__(self):
        self.loadConfig()
        self.output_all()

    def loadConfig(self):
        print '初始化...'
        with open('config.json') as f:
            self.block_lst += [s.encode('utf8') for s in json.loads(f.readline())]
            self.to_play_lst += [s.encode('utf8') for s in json.loads(f.readline())]
            self.pending_lst += [s.encode('utf8') for s in json.loads(f.readline())]
        print '初始化完成'

    def saveConfig(self):
        print '保存数据...'
        with open('config.json', 'w') as f:
            self.block_lst = list(set(self.block_lst))
            f.write(json.dumps(self.block_lst) + '\n')
            f.write(json.dumps(self.to_play_lst) + '\n')
            f.write(json.dumps(self.pending_lst) + '\n')
        print '保存数据完成'

    def clear(self): 
        print("\033c")

    def output_help(self):
        print('操作指南: f-收藏,b-屏蔽,c-已弹奏,d-删除,df-删除收藏,s-显示详细列表,q-退出')

    def output_all(self):
        self.clear()
        print '当前待演奏列表：'
        for i, s in enumerate(self.to_play_lst):
            print '%d.\t%s' % (i+1, s)
        print '--------------------------------'
        print '当前收藏列表：'
        for i, s in enumerate(self.pending_lst):
            print '%d.\t%s' % (i+1, s)
        print '--------------------------------'
        print '已弹奏列表：'
        for k in self.completed.keys():
            print k, '\t', self.completed[k], '次'
        print '--------------------------------'
        self.output_help()

    def output(self):
        self.clear()
        print '当前待演奏列表：'
        for i, s in enumerate(self.to_play_lst):
            print '%d.\t%s' % (i+1, s)

    def handleDanmaku(self, danmaku):
        body = danmaku.rawData
        if danmaku.action == 5: # danmaku packet
            raw = json.loads(body)
            if 'info' in raw:
                info = raw['info']
                s = info[1].encode('utf-8').strip()
                if s[:9] in ['点歌机', '点歌姬', '點歌機', '點歌姬']: return
                if s[:6] in ['点歌', '點歌']: content = s[6:].strip()
                else: return
                if any([1 for keyword in self.block_lst if keyword in content]): return
                if content in self.pending_lst or content in self.to_play_lst: return
                self.to_play_lst += [content]
                self.output()

    def getIdx(self, s, offset):
        try:
            idx = s[offset:]
            if not idx: idx = 0
            else: idx = int(idx) - 1
            return idx
        except Exception, e:
            if DEBUG: print 'error!', e
            return 0

    def parseCommand(self, s):
        try:
            s = s.strip()
            if s == 's': 
                pass
            elif s[:2] == 'df':
                idx = self.getIdx(s, 2)
                if idx < len(self.pending_lst):
                    self.pending_lst.pop(idx)
            elif s[0] == 'd': 
                idx = self.getIdx(s, 1)
                if idx < len(self.to_play_lst):
                    self.to_play_lst.pop(idx)
            elif s[0] == 'b': 
                idx = self.getIdx(s, 1)
                if idx < len(self.to_play_lst):
                    content = self.to_play_lst[idx] 
                    self.block_lst += [content]
                    self.to_play_lst.pop(idx)
            elif s[0] == 'f':
                idx = self.getIdx(s, 1)
                if idx < len(self.to_play_lst):
                    content = self.to_play_lst[idx] 
                    self.pending_lst += [content]
                    self.to_play_lst.pop(idx)
            elif s[0] == 'c':
                idx = self.getIdx(s, 1)
                if idx < len(self.to_play_lst):
                    content = self.to_play_lst[idx] 
                    self.completed[content] = self.completed.get(content, 0) + 1
                    self.to_play_lst.pop(idx)
            elif s[0] == 'q':
                self.saveConfig()
                self.output_all()
                sys.exit(0)
                return
            self.output_all()
        except Exception, e:
            if DEBUG: print 'error!', e

if __name__ == '__main__':
    argv = sys.argv
    roomid = 30040
    if len(argv) == 2:
        roomid = int(argv[1])

    danmakuHandler = DanmakuHandler()
    py = bili.BiliHelper(roomid, danmakuHandler)
    while 1:
        cmd = raw_input()
        if cmd == '': danmakuHandler.output_all()
        elif cmd[0] in 'sbdfrcq': danmakuHandler.parseCommand(cmd)
        else: danmakuHandler.handleDanmaku(cmd)



#!/usr/bin/python
#coding=utf-8
import bili
import bili_sender
import bili_config
import struct
import time
import json
import sys
import signal

from handlers import *

DEBUG = 0

class DanmakuHandler(bili.DanmakuHandler):
    #log = open('/Users/kliner/blog/' + time.strftime('%d-%H%M%S', time.localtime(time.time() + 18000)) + '.log', 'w')

    def __init__(self, roomid, config):
        self.cnt = 9
        self.showTime = config.get(roomid, "ShowTime", False)
        #self.color = config.get(roomid, "DanmakuColor", "white")
    
        print '----------------------------'
        print 'showTime...', self.showTime 
        #print 'danmakuColor...', self.color
        self.date_format = '%H:%M:%S'
            
        signal.signal(signal.SIGINT, self.signal_handler)

    def handleDanmaku(self, danmaku):
        body = danmaku.rawData
        if danmaku.action == 3: # online clients
            onlineNum = struct.unpack('>i', body)[0]
            self.cnt += 1
            if self.cnt >= 10: 
                print '在线 \033[91m♥︎\033[0m :' + str(onlineNum)
                self.cnt = 0
        elif danmaku.action == 5: # danmaku packet
            raw = json.loads(body)
            if DEBUG: print raw

            tm = time.strftime(self.date_format, danmaku.time)
            if 'info' in raw:
                info = raw['info']
                user, content = info[2][1].encode('utf-8'), info[1].encode('utf-8')
                if self.showTime:
                    print '[%s] \033[91m%s\033[0m : \033[94m%s\033[0m' % (tm, user, content)
                else:
                    print '\033[91m%s\033[0m : \033[94m%s\033[0m' % (user, content)

            elif raw['cmd'] == 'SEND_GIFT': pass
            elif raw['cmd'] == 'WELCOME': pass 
            elif raw['cmd'] == 'WELCOME_GUARD': pass 
            elif raw['cmd'] in ['SYS_GIFT']: pass
            elif raw['cmd'] in ['SYS_MSG']: pass
            else: 
                if DEBUG: print raw
        else:
            if DEBUG: print 'unknown action,' + repr(danmaku) 
    
    def signal_handler(self, signal, frame):
        print('exit... saving info...')
        #self.log.close()
        sys.exit(0)

def initHandlers(roomid):
    config = bili_config.Config()
    cookies = config.cookies

    handler = DanmakuHandler(roomid, config = config)
    sender = bili_sender.Sender(cookies)

    danmakuHandlers = [handler]
    if config.get(roomid, "RecordDanmaku", False): danmakuHandlers += [MongoHandler()]
    if config.get(roomid, "MacTTS", False): danmakuHandlers += [TTSHandler()]
    if config.get(roomid, "MacNotification", False): danmakuHandlers += [NotifcationHandler()]
    if config.get(roomid, "GiftResponse", False): danmakuHandlers += [GiftResponseHandler(sender)]
    if config.get(roomid, "AwardSmallTV", False): danmakuHandlers += [AutoRewardHandler(sender)]
    if config.get(roomid, "SmallTVHint", False): danmakuHandlers += [RewardResponseHandler(sender)]
    return danmakuHandlers

def main():
    argv = sys.argv
    roomid = 90012
    if len(argv) == 2: roomid = int(argv[1])

    py = bili.BiliHelper(roomid, *initHandlers(roomid))
    
    config = bili_config.Config()
    cookies = config.cookies
    sender = bili_sender.Sender(cookies)
    while 1:
        cmd = raw_input()
        sender.sendDanmaku(roomid, cmd)

if __name__ == '__main__':
    main()



#!/usr/bin/python
#coding=utf-8
import bili
import bili_sender
import bili_config
import sys
import thread
import struct
import time
import json
import os
import mplayer
import random
import threading
import subprocess

import traceback
import pymongo
from pymongo import MongoClient
from bson.objectid import ObjectId

reload(sys)  
sys.setdefaultencoding('utf8')

DEBUG = 0
DIVIDER = '--------------------'

class Music(object):

    def __init__(self, n, s, e): 
        self.name, self.sname, self.ename = n, s, e

    def __str__(self): 
        return self.searchKey()

    def __repr__(self): 
        return self.__str__()
    
    def searchKey(self): 
        return '%s %s %s' % (self.name, self.sname, self.ename)

class DBHelper(object):

    def __init__(self):
        try:
            print 'start db...'
            self.client = MongoClient()
            self.db = self.client.music
        except:
            print 'mongodb service down!\nto install:[brew install mongodb]\nto start:[brew services start mongodb]'
            self.db = None

    def selectFavorite(self, danmaku):
        if self.db:
            cursor = self.db.fav.find({'user': danmaku.user}).sort([('time', pymongo.ASCENDING)])
            return [d['name'] for d in cursor]

    def removeFavorite(self, danmaku, name):
        if self.db:
            cursor = self.db.fav.find({'user': danmaku.user}).sort([('time', pymongo.DESCENDING)])
            i = 0
            for d in cursor:
                i += 1
                if i >= 9:
                    try:
                        self.db.fav.delete_one({'_id': ObjectId(d['_id'])})
                    except:
                        if DEBUG: traceback.print_exc()

    def insertFavorite(self, danmaku, name):
        if self.db:
            favs = self.selectFavorite(danmaku)
            if DEBUG: print favs
            if len(favs) >= 9:
                self.removeFavorite(danmaku, name)

            self.db.fav.insert_one({
                'user':danmaku.user,
                'name':name,
                'time':time.strftime('%y%m%d-%H%M%S', danmaku.time)
                })
            return self.selectFavorite(danmaku)

class DanmakuHandler(bili.SimpleDanmakuHandler):
    home_path = os.path.expanduser("~")
    lib_path = home_path + '/Music/lib/'
    all_music = []
    to_play_lst = []
    p = mplayer.Player()
    LOCK = threading.Lock()
    cur_user = None
    state = 'play'
    timer = None
    config = bili_config.Config()
    sender = bili_sender.Sender(config.cookies)
    skip = False

    def __init__(self, roomid):
        self.roomid = roomid
        self.loadMusic()
        thread.start_new_thread(self.musicThread, ())
        self.db = DBHelper()

    def clear(self): 
        print("\033c")
        if self.p.filename: print '正在播放: ', self.p.filename[:-4]

    def printHelp(self): print('发 \'点歌\' 进入 点歌模式，在点歌模式下发 \'搜索 关键字\' 搜索列表，在点歌模式下发送 \'点歌 ID\' 完成点歌。发送 \'退出\' 结束点歌。请在五分钟内完成全部操作哦～')

    def printToPlay(self):
        print '当前待播放列表：'
        for u, m in self.to_play_lst: print '%s 点了\t: %s' % (u, m.name) 

    def loadMusic(self):
        origin_music = [f[:-4] for f in os.listdir(self.lib_path) if f[-4:] == '.mp3']
        with open('%s/.pybili.ti' % self.home_path, 'w') as f:
            f.write('\n'.join(origin_music))
        
        subprocess.Popen('opencc -i %s/.pybili.ti -o %s/.pybili.to -c t2s.json' % (self.home_path, self.home_path), shell=True)

        with open('%s/.pybili.to' % self.home_path, 'r') as f:
            lst = f.read().split('\n')
            self.all_music = [Music(n,s,n) for n, s in zip(origin_music, lst)]
           
        if DEBUG: print self.all_music
        if DEBUG: print self.all_music[0], len(self.all_music)
        
    def playMusic(self, name):
        if DEBUG: 
            print 'player state:', self.p.is_alive()
            print 'self state:', self.state
        while self.state != 'play': time.sleep(1)
        self.p.stop()
        self.p.loadfile(self.lib_path + name + '.mp3')
        time.sleep(0.5)
        if DEBUG: print 'playing ', self.p.filename
        self.clear()
        if self.cur_user: print '当前操作者：', self.cur_user
        self.printToPlay()

        self.p.volume = 8
        length = self.p.length
        self.p.pause()
        if DEBUG: print length
        for i in xrange(int(length)):
            if self.skip:
                if DEBUG: print 'skip play ', name, self.p.filename
                self.p.stop()
                self.skip = False
                break
            time.sleep(1)
        else:
            if DEBUG: print 'finish play ', name, self.p.filename


    def musicThread(self):
        while 1:
            if self.to_play_lst:
                self.LOCK.acquire()
                u, music = self.to_play_lst.pop(0)
                self.LOCK.release()
                self.playMusic(music.name)
            else:
                music = random.choice(self.all_music)
                self.playMusic(music.name)
            if not self.cur_user:
                self.printToPlay()

    def localTimerThread(self, user):
        if self.cur_user == user: 
            self.cur_user = None
            self.clear()
            self.printToPlay()
            print '五分钟到了哦～'

    def match(self, key, music):
        if key in music.lower(): return True
        keys = key.split(' ')
        if len(keys) > 1: 
            if all(k in music.lower() for k in keys): return True

    def search(self, key):
        result = []
        for i, m in enumerate(self.all_music):
            if self.match(key, m.searchKey()): result += [(i+1, m.name)]
        if len(result) == 0: 
            self.sender.sendDanmaku(self.roomid, 'Sorry...这里没有对应的歌')
        elif len(result) == 1:
            self.addToPlayList(result[0][0])
        else:
            if self.cur_user != 'klikli': self.sender.sendDanmaku(self.roomid, '搜索 %s 中...' % key)
            print '搜索 %s 的结果列表：' % key 
            for i, t in result:
                print '%d\t: %s' % (i, t)
            print '切歌时候会导致搜索结果丢失，请注意重新搜索哦'
        
    def addToPlayListByName(self, name):
        to_add = Music(name, name, name)
        self.LOCK.acquire()
        if len(self.to_play_lst) < 10:
            if not any([1 for _, music in self.to_play_lst if music.name == to_add.name]):
                self.to_play_lst += [(self.cur_user, to_add)]
        self.LOCK.release()
        self.clear()
        self.printToPlay()
        self.sender.sendDanmaku(self.roomid, '[%s...]点歌成功' % to_add.name[:15])

    def addToPlayList(self, i):
        to_add = self.all_music[i-1]
        self.addToPlayListByName(to_add.name)

    def printFav(self, danmaku):
        print DIVIDER
        print '%s的收藏列表: ' % danmaku.user
        favs = self.db.selectFavorite(danmaku)
        for i, name in enumerate(favs):
            print '%d, %s' % (i+1, name)

    def handleDanmaku(self, danmaku):
        super(DanmakuHandler, self).handleDanmaku(danmaku)
        body = danmaku.rawData
        if danmaku.action == 5:
            raw = json.loads(body)
            if DEBUG: print raw
            if 'info' in raw:
                info = raw['info']
                user = info[2][1].encode('utf-8')
                manager = info[2][2]
                content = info[1].encode('utf-8')
                if manager and content in ['切歌']:
                    self.skip = True

                if content in ['查看收藏','收藏列表']:
                    if (not self.cur_user) or (self.cur_user == user):
                        self.clear()
                        self.printToPlay()
                        self.printFav(danmaku)
                elif content[:6] in ['收藏']:
                    try:
                        if len(content) == 6: to_add = self.p.filename[:-4]
                        else: 
                            i = int(content[6:].strip())
                            to_add = self.all_music[i-1].name
                        self.db.insertFavorite(danmaku, to_add)
                        self.sender.sendDanmaku(self.roomid, '[%s...]收藏成功' % to_add[:15])
                    except Exception, e:
                        if DEBUG: traceback.print_exc()
                        self.sender.sendDanmaku(self.roomid, '请输入正确的指令哦')
                elif content[:6] in ['点歌', '點歌']: 
                    if not self.cur_user:
                        self.cur_user = user
                        self.printHelp()
                        print '当前操作者：' + user
                        self.timer = threading.Timer(300, self.localTimerThread, (user, ))
                        self.timer.start()
                        self.sender.sendDanmaku(self.roomid, '%s开始点歌～' % self.cur_user)
                    if self.cur_user == user:
                        k = content[6:].strip()
                        if not k: return
                        try:
                            if k[0] == '@': # play from favorite
                                i = int(k[1:])
                                favs = self.db.selectFavorite(danmaku)
                                self.addToPlayListByName(favs[i-1])
                            else:
                                if k.isdigit(): self.addToPlayList(int(k)) # play by id
                                else: self.search(k.lower()) # search
                        except Exception, e:
                            if DEBUG: print e
                            self.sender.sendDanmaku(self.roomid, '请输入正确的点歌指令哦')
                    else:
                        self.sender.sendDanmaku(self.roomid, '%s正在点歌, 请等一下哦' % self.cur_user)
                elif user == self.cur_user and content[:6] in ['搜索']:
                    self.clear()
                    key = content[6:].strip().lower()
                    self.search(key)
                elif user == self.cur_user and content.lower() in ['退出', 'exit', '结束', 'quit']: 
                    self.sender.sendDanmaku(self.roomid, '欢迎再来点歌哦～')
                    self.cur_user = None
                    if self.timer: self.timer.cancel()
                    self.clear()
                    self.printToPlay()
                elif user == 'klikli' and content == 'reload':
                    self.loadMusic()
                    print '重新加载歌曲库...'

def main():
    argv = sys.argv
    roomid = 90012
    if len(argv) == 2:
        roomid = int(argv[1])

    danmakuHandler = DanmakuHandler(roomid)
    py = bili.BiliHelper(roomid, danmakuHandler)
    while 1:
        cmd = raw_input().strip()
        if cmd == 'p':
            danmakuHandler.p.pause()
            if danmakuHandler.state == 'pause': 
                danmakuHandler.state = 'play'
                print 'play'
            elif danmakuHandler.state == 'play': 
                danmakuHandler.state = 'pause'
                print 'pause'
        elif cmd == 'r':
            danmakuHandler.loadMusic()
        else:
            danmakuHandler.search(cmd)

                        
if __name__ == '__main__':
    main()



from distutils.core import setup

setup(
  name = 'pybili',
  packages = ['pybili'], # this must be the same as the name above
  version = '0.2.4',
  description = 'A helper library for bilibili.com',
  author = 'Kliner',
  author_email = 'kliner@live.cn',
  url = 'https://github.com/kliner/pyBili', # use the URL to the github repo
  download_url = 'https://github.com/kliner/pyBili/archive/0.2.3.tar.gz',
  keywords = ['bili', 'live', 'danmaku'], # arbitrary keywords
  classifiers = [],

  install_requires=['requests', 'pymongo', 'emoji_list'],

  entry_points={
    'console_scripts':[
      'bili-danmuji = pybili.GUI:main',
      'bili-dm = pybili.DanMuJi:main',
      'bili-music = pybili.AutoDianGeJi:main',
      'bili-sender = pybili.bili_sender:main',
      'bili-config = pybili.bili_config:main',
    ]
  }
)
